@@ -1,7 +1,7 @@
 /** @file wordaccess.h
  * @brief functions for reading and writing different width words
  */
-/* Copyright (C) 2016,2018 Olly Betts
+/* Copyright (C) 2016,2018,2019 Olly Betts
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -37,7 +37,7 @@
 # include <stdlib.h>
 #endif
 
-inline int do_bswap(uint16_t value) {
+inline uint16_t do_bswap(uint16_t value) {
 # if HAVE_DECL___BUILTIN_BSWAP16
     return __builtin_bswap16(value);
 # elif HAVE_DECL__BYTESWAP_USHORT
@@ -47,10 +47,10 @@ inline int do_bswap(uint16_t value) {
 # endif
 }
 
-inline int do_bswap(uint32_t value) {
+inline uint32_t do_bswap(uint32_t value) {
 # if HAVE_DECL___BUILTIN_BSWAP32
     return __builtin_bswap32(value);
-# elif HAVE_DECL__BYTESWAP_USHORT
+# elif HAVE_DECL__BYTESWAP_ULONG
     return _byteswap_ulong(value);
 # else
     return (value << 24) |
@@ -60,6 +60,23 @@ inline int do_bswap(uint32_t value) {
 # endif
 }
 
+inline uint64_t do_bswap(uint64_t value) {
+# if HAVE_DECL___BUILTIN_BSWAP64
+    return __builtin_bswap64(value);
+# elif HAVE_DECL__BYTESWAP_UINT64
+    return _byteswap_uint64(value);
+# else
+    return (value << 56) |
+	   ((value & 0xff00) << 40) |
+	   ((value & 0xff0000) << 24) |
+	   ((value & 0xff000000) << 8) |
+	   ((value >> 8) & 0xff000000) |
+	   ((value >> 24) & 0xff0000) |
+	   ((value >> 40) & 0xff00) |
+	   (value >> 56);
+# endif
+}
+
 #endif
 
 template<typename UINT>
