@@ -1,7 +1,7 @@
 /** @file queryparser.h
  * @brief parsing a user query string to build a Xapian::Query object
  */
-/* Copyright (C) 2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017 Olly Betts
+/* Copyright (C) 2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Olly Betts
  * Copyright (C) 2010 Adam Sjøgren
  *
  * This program is free software; you can redistribute it and/or
@@ -109,7 +109,7 @@ class XAPIAN_VISIBILITY_DEFAULT SimpleStopper : public Stopper {
      * @endcode
      *
      */
-    template <class Iterator>
+    template<class Iterator>
     SimpleStopper(Iterator begin, Iterator end) : stop_words(begin, end) { }
 
     /// Add a single stop word.
@@ -205,7 +205,10 @@ class XAPIAN_VISIBILITY_DEFAULT RangeProcessor
      *			by the user (empty string for no upper limit).
      *
      *  @return		An OP_VALUE_RANGE Query object (or if end.empty(), an
-     *			OP_VALUE_GE Query object).
+     *			OP_VALUE_GE Query object).  Or if the range isn't one
+     *			which this object can handle then
+     *			Xapian::Query(Xapian::Query::OP_INVALID) will be
+     *			returned.
      */
     virtual Xapian::Query
 	operator()(const std::string &begin, const std::string &end);
@@ -884,7 +887,9 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
     } feature_flag;
 
     /// Stemming strategies, for use with set_stemming_strategy().
-    typedef enum { STEM_NONE, STEM_SOME, STEM_ALL, STEM_ALL_Z } stem_strategy;
+    typedef enum {
+	STEM_NONE, STEM_SOME, STEM_ALL, STEM_ALL_Z, STEM_SOME_FULL_POS
+    } stem_strategy;
 
     /// Copy constructor.
     QueryParser(const QueryParser & o);
@@ -892,6 +897,14 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
     /// Assignment.
     QueryParser & operator=(const QueryParser & o);
 
+#ifdef XAPIAN_MOVE_SEMANTICS
+    /// Move constructor.
+    QueryParser(QueryParser && o);
+
+    /// Move assignment operator.
+    QueryParser & operator=(QueryParser && o);
+#endif
+
     /// Default constructor.
     QueryParser();
 
@@ -915,8 +928,8 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
     /** Set the stemming strategy.
      *
      *  This controls how the query parser will apply the stemming algorithm.
-     *  Note that the stemming algorithm is only applied to words in
-     *  probabilistic fields - boolean filter terms are never stemmed.
+     *  Note that the stemming algorithm is only applied to words in free-text
+     *  fields - boolean filter terms are never stemmed.
      *
      *  @param strategy	The strategy to use - possible values are:
      *   - STEM_NONE:	Don't perform any stemming.  (default in Xapian <=
@@ -927,6 +940,10 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *			with operators which need positional information.
      *			Stemmed terms are prefixed with 'Z'.  (default in
      *			Xapian >= 1.3.1)
+     *   - STEM_SOME_FULL_POS:
+     *			Like STEM_SOME but also stems terms used with operators
+     *			which need positional information.  Added in Xapian
+     *			1.4.8.
      *   - STEM_ALL:	Stem all terms (note: no 'Z' prefix is added).
      *   - STEM_ALL_Z:	Stem all terms (note: 'Z' prefix is added).  (new in
      *			Xapian 1.2.11 and 1.3.1)
@@ -1038,7 +1055,7 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 		      unsigned flags = FLAG_DEFAULT,
 		      const std::string &default_prefix = std::string());
 
-    /** Add a probabilistic term prefix.
+    /** Add a free-text field term prefix.
      *
      *  For example:
      *
@@ -1093,7 +1110,7 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *  @endcode
      *
      *  This allows the user to restrict a search with site:xapian.org which
-     *  will be converted to Hxapian.org combined with any probabilistic
+     *  will be converted to Hxapian.org combined with any weighted
      *  query with @c Xapian::Query::OP_FILTER.
      *
      *  If multiple boolean filters are specified in a query for the same
