@@ -364,16 +364,16 @@ DEFINE_TESTCASE(testlock1, chert || glass) {
 	TEST(db.locked());
 	TEST(rdb.locked());
 	// After close(), locked() should either work as if close() hadn't been
-	// called or throw Xapian::DatabaseError.
+	// called or throw Xapian::DatabaseClosedError.
 	try {
 	    TEST(db_as_database.locked());
-	} catch (const Xapian::DatabaseError&) {
+	} catch (const Xapian::DatabaseClosedError&) {
 	}
 	db.close();
 	TEST(!rdb.locked());
 	try {
 	    TEST(!db_as_database.locked());
-	} catch (const Xapian::DatabaseError&) {
+	} catch (const Xapian::DatabaseClosedError&) {
 	}
     }
     TEST(!rdb.locked());
@@ -976,9 +976,9 @@ DEFINE_TESTCASE(emptydb1, backend) {
 /// Test error opening non-existent stub databases.
 // Regression test for bug fixed in 1.3.1 and 1.2.11.
 DEFINE_TESTCASE(stubdb7, !backend) {
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
 	    Xapian::Database("nosuchdirectory", Xapian::DB_BACKEND_STUB));
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
 	    Xapian::WritableDatabase("nosuchdirectory",
 		Xapian::DB_OPEN|Xapian::DB_BACKEND_STUB));
     return true;
@@ -1197,7 +1197,7 @@ DEFINE_TESTCASE(phrasebug1, generated && positional) {
 }
 
 /// Feature test for Xapian::DB_RETRY_LOCK
-DEFINE_TESTCASE(retrylock1, writable && !inmemory && !remote) {
+DEFINE_TESTCASE(retrylock1, writable && path) {
     // FIXME: Can't see an easy way to test this for remote databases - the
     // harness doesn't seem to provide a suitable way to reopen a remote.
 #if defined HAVE_FORK && defined HAVE_SOCKETPAIR
@@ -1205,6 +1205,8 @@ DEFINE_TESTCASE(retrylock1, writable && !inmemory && !remote) {
     if (socketpair(AF_UNIX, SOCK_STREAM|SOCK_CLOEXEC, PF_UNSPEC, fds) < 0) {
 	FAIL_TEST("socketpair() failed");
     }
+    if (fds[1] >= FD_SETSIZE)
+	SKIP_TEST("socketpair() gave fd >= FD_SETSIZE");
     if (fcntl(fds[1], F_SETFL, O_NONBLOCK) < 0)
 	FAIL_TEST("fcntl() failed to set O_NONBLOCK");
     pid_t child = fork();
@@ -1275,12 +1277,10 @@ retry:
 	struct timeval tv;
 	tv.tv_sec = 3;
 	tv.tv_usec = 0;
-	fd_set fr, fe;
-	FD_ZERO(&fr);
-	FD_SET(fds[1], &fr);
-	FD_ZERO(&fe);
-	FD_SET(fds[1], &fe);
-	int sr = select(fds[1] + 1, &fr, NULL, &fe, &tv);
+	fd_set fdset;
+	FD_ZERO(&fdset);
+	FD_SET(fds[1], &fdset);
+	int sr = select(fds[1] + 1, &fdset, NULL, NULL, &tv);
 	if (sr == 0) {
 	    // Timed out.
 	    result[0] = 'T';
