@@ -1,7 +1,7 @@
 /* api_replicate.cc: tests of replication functionality
  *
  * Copyright 2008 Lemur Consulting Ltd
- * Copyright 2009,2010,2011,2012,2013,2014,2015,2016 Olly Betts
+ * Copyright 2009,2010,2011,2012,2013,2014,2015,2016,2017 Olly Betts
  * Copyright 2010 Richard Boulton
  * Copyright 2011 Dan Colish
  *
@@ -33,7 +33,6 @@
 #include "fd.h"
 #include "filetests.h"
 #include "safedirent.h"
-#include "safeerrno.h"
 #include "safefcntl.h"
 #include "safesysstat.h"
 #include "safeunistd.h"
@@ -43,13 +42,16 @@
 
 #include <sys/types.h>
 
+#include <cerrno>
 #include <cstdlib>
 #include <string>
 
-#include <stdlib.h> // For setenv() or putenv()
+#include <stdlib.h> // For setenv() or putenv() or _putenv_s()
 
 using namespace std;
 
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
+
 static void rmtmpdir(const string & path) {
     rm_rf(path);
 }
@@ -236,7 +238,7 @@ check_equal_dbs(const string & masterpath, const string & replicapath)
 #if 0 // Dynamic version which we don't currently need.
 static void
 set_max_changesets(int count) {
-#ifdef HAVE__PUTENV_S
+#if HAVE_DECL__PUTENV_S
     _putenv_s("XAPIAN_MAX_CHANGESETS", str(count).c_str());
 #elif defined HAVE_SETENV
     setenv("XAPIAN_MAX_CHANGESETS", str(count).c_str(), 1);
@@ -248,7 +250,7 @@ set_max_changesets(int count) {
 }
 #endif
 
-#ifdef HAVE__PUTENV_S
+#if HAVE_DECL__PUTENV_S
 # define set_max_changesets(N) _putenv_s("XAPIAN_MAX_CHANGESETS", #N)
 #elif defined HAVE_SETENV
 # define set_max_changesets(N) setenv("XAPIAN_MAX_CHANGESETS", #N, 1)
@@ -265,11 +267,14 @@ struct unset_max_changesets_helper_ {
 // testcase, even if this one exits with an exception.
 #define UNSET_MAX_CHANGESETS_AFTERWARDS unset_max_changesets_helper_ ezlxq
 
+#endif
+
 // #######################################################################
 // # Tests start here
 
 // Basic test of replication functionality.
 DEFINE_TESTCASE(replicate1, replicas) {
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
     UNSET_MAX_CHANGESETS_AFTERWARDS;
     string tempdir = ".replicatmp";
     mktmpdir(tempdir);
@@ -341,11 +346,13 @@ DEFINE_TESTCASE(replicate1, replicas) {
     }
 
     rmtmpdir(tempdir);
+#endif
     return true;
 }
 
 // Test replication from a replicated copy.
 DEFINE_TESTCASE(replicate2, replicas) {
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
     SKIP_TEST_FOR_BACKEND("glass"); // Glass doesn't currently support this.
     UNSET_MAX_CHANGESETS_AFTERWARDS;
 
@@ -443,9 +450,11 @@ DEFINE_TESTCASE(replicate2, replicas) {
     }
 
     rmtmpdir(tempdir);
+#endif
     return true;
 }
 
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
 static void
 replicate_with_brokenness(Xapian::DatabaseMaster & master,
 			  Xapian::DatabaseReplica & replica,
@@ -485,9 +494,11 @@ replicate_with_brokenness(Xapian::DatabaseMaster & master,
 	}
     }
 }
+#endif
 
 // Test changesets which are truncated (and therefore invalid).
 DEFINE_TESTCASE(replicate3, replicas) {
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
     UNSET_MAX_CHANGESETS_AFTERWARDS;
     string tempdir = ".replicatmp";
     mktmpdir(tempdir);
@@ -533,11 +544,13 @@ DEFINE_TESTCASE(replicate3, replicas) {
     }
 
     rmtmpdir(tempdir);
+#endif
     return true;
 }
 
 // Tests for max_changesets
 DEFINE_TESTCASE(replicate4, replicas) {
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
     UNSET_MAX_CHANGESETS_AFTERWARDS;
     string tempdir = ".replicatmp";
     mktmpdir(tempdir);
@@ -630,12 +643,13 @@ DEFINE_TESTCASE(replicate4, replicas) {
     }
 
     rmtmpdir(tempdir);
+#endif
     return true;
 }
 
-
 // Tests for max_changesets
 DEFINE_TESTCASE(replicate5, replicas) {
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
     SKIP_TEST_FOR_BACKEND("chert");
     UNSET_MAX_CHANGESETS_AFTERWARDS;
     string tempdir = ".replicatmp";
@@ -762,11 +776,13 @@ DEFINE_TESTCASE(replicate5, replicas) {
     }
 
     rmtmpdir(tempdir);
+#endif
     return true;
 }
 
 /// Test --full-copy option.
 DEFINE_TESTCASE(replicate6, replicas) {
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
     UNSET_MAX_CHANGESETS_AFTERWARDS;
     string tempdir = ".replicatmp";
     mktmpdir(tempdir);
@@ -827,11 +843,13 @@ DEFINE_TESTCASE(replicate6, replicas) {
     }
 
     rmtmpdir(tempdir);
+#endif
     return true;
 }
 
 /// Test healing a corrupt replica (new in 1.3.5).
 DEFINE_TESTCASE(replicate7, replicas) {
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
     UNSET_MAX_CHANGESETS_AFTERWARDS;
     string tempdir = ".replicatmp";
     mktmpdir(tempdir);
@@ -908,5 +926,6 @@ DEFINE_TESTCASE(replicate7, replicas) {
     }
 
     rmtmpdir(tempdir);
+#endif
     return true;
 }
