@@ -125,7 +125,20 @@ static bool test_refcnt2()
 
     Xapian::Internal::intrusive_ptr<test_refcnt> rcp(p);
 
+#ifdef __has_warning
+# if __has_warning("-Wself-assign-overloaded")
+    // Suppress warning from newer clang about self-assignment so we can
+    // test that self-assignment works!
+#  pragma clang diagnostic push
+#  pragma clang diagnostic ignored "-Wself-assign-overloaded"
+# endif
+#endif
     rcp = rcp;
+#ifdef __has_warning
+# if __has_warning("-Wself-assign-overloaded")
+#  pragma clang diagnostic pop
+# endif
+#endif
 
     TEST_AND_EXPLAIN(!deleted, "Object deleted by self-assignment");
 
@@ -518,7 +531,7 @@ static bool test_chartype1()
     }
 
     // Non-ASCII characters aren't anything for these functions.
-    for (int ch = 128; ch != 256; ++ch) {
+    for (unsigned char ch = 128; ch != 0; ++ch) {
 	TEST(!C_isupper(ch));
 	TEST(!C_islower(ch));
 	TEST(!C_isalpha(ch));
@@ -536,7 +549,7 @@ static bool test_chartype1()
     }
 
     // Check signed char values work the same way.
-    for (int ch = -128; ch != 0; ++ch) {
+    for (signed char ch = -128; ch != 0; ++ch) {
 	TEST(!C_isupper(ch));
 	TEST(!C_islower(ch));
 	TEST(!C_isalpha(ch));
