@@ -1,7 +1,7 @@
 /** @file unittest.cc
  * @brief Unit tests of non-Xapian-specific internal code.
  */
-/* Copyright (C) 2006,2007,2009,2010,2012,2015,2016 Olly Betts
+/* Copyright (C) 2006,2007,2009,2010,2012,2015,2016,2018 Olly Betts
  * Copyright (C) 2007 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
@@ -22,6 +22,7 @@
 
 #include <config.h>
 
+#include <cctype>
 #include <cfloat>
 #include <climits>
 #include <cmath>
@@ -73,13 +74,21 @@ using namespace std;
 #include "../common/closefrom.cc"
 #include "../common/errno_to_string.cc"
 #include "../common/fileutils.cc"
+#include "../common/overflow.h"
 #include "../common/serialise-double.cc"
 #include "../common/str.cc"
+#include "../backends/uuids.cc"
 #include "../net/length.cc"
 #include "../net/serialise-error.cc"
 #include "../api/error.cc"
 #include "../api/sortable-serialise.cc"
 
+// fileutils.cc uses opendir(), etc though not in a function we currently test.
+#include "../common/msvc_dirent.cc"
+
+// The UUID code uses hexdigit().
+#include "../api/constinfo.cc"
+
 // Stub replacement, which doesn't deal with escaping or producing valid UTF-8.
 // The full implementation needs Xapian::Utf8Iterator and
 // Xapian::Unicode::append_utf8().
@@ -110,7 +119,7 @@ DEFINE_TESTCASE_(class_exceptions_work1) {
     return false;
 }
 
-inline string
+static inline string
 r_r_p(string a, const string & b)
 {
     resolve_relative_path(a, b);
@@ -575,6 +584,203 @@ static bool test_closefrom1()
     return true;
 }
 
+static bool test_uuid1()
+{
+    Uuid uuid, uuid2;
+
+    // Test a generated uuid.
+    uuid.generate();
+    TEST(!uuid.is_null());
+    string str = uuid.to_string();
+    TEST_EQUAL(str.size(), 36);
+    TEST_NOT_EQUAL(str, "00000000-0000-0000-0000-000000000000");
+    // Check UUID pattern is correct and that upper case is not used.
+    for (int i = 0; i != 8; ++i) {
+	unsigned char ch = str[i];
+	TEST(isxdigit(ch));
+	TEST(!isupper(ch));
+    }
+    TEST_EQUAL(str[8], '-');
+    for (int i = 9; i != 13; ++i) {
+	unsigned char ch = str[i];
+	TEST(isxdigit(ch));
+	TEST(!isupper(ch));
+    }
+    TEST_EQUAL(str[13], '-');
+    for (int i = 14; i != 18; ++i) {
+	unsigned char ch = str[i];
+	TEST(isxdigit(ch));
+	TEST(!isupper(ch));
+    }
+    TEST_EQUAL(str[18], '-');
+    for (int i = 19; i != 23; ++i) {
+	unsigned char ch = str[i];
+	TEST(isxdigit(ch));
+	TEST(!isupper(ch));
+    }
+    TEST_EQUAL(str[23], '-');
+    for (int i = 24; i != 36; ++i) {
+	unsigned char ch = str[i];
+	TEST(isxdigit(ch));
+	TEST(!isupper(ch));
+    }
+
+    uuid2.parse(str);
+    TEST(memcmp(uuid.data(), uuid2.data(), uuid.BINARY_SIZE) == 0);
+
+    // Check the variant is "10x" and the version between 1 and 5.  Mostly this
+    // is to catch bugs where the platform's API for generating UUIDs uses a
+    // different endianness for fields (which we've run into under both WIN32
+    // and FreeBSD).
+    TEST_EQUAL(uuid.data()[8] & 0xc0, 0x80);
+    TEST_REL(str[19], >=, '8');
+    TEST_REL(str[19], <=, 'b');
+    TEST_REL(int(uuid.data()[6]), >=, 0x10);
+    TEST_REL(int(uuid.data()[6]), <=, 0x5f);
+    TEST_REL(str[14], >=, '1');
+    TEST_REL(str[14], <=, '5');
+
+    // Test generating another uuid gives us a different non-null uuid.
+    uuid2.generate();
+    TEST(!uuid2.is_null());
+    TEST(memcmp(uuid.data(), uuid2.data(), uuid.BINARY_SIZE) != 0);
+
+    // Test null uuid.
+    uuid.clear();
+    TEST(uuid.is_null());
+    str = uuid.to_string();
+    TEST_EQUAL(str, "00000000-0000-0000-0000-000000000000");
+    uuid2.generate();
+    TEST(!uuid2.is_null());
+    uuid2.parse(str);
+    TEST(memcmp(uuid.data(), uuid2.data(), uuid.BINARY_SIZE) == 0);
+
+    return true;
+}
+
+// Classes used by movesupport1 test
+class A : public Xapian::Internal::intrusive_base {
+    int x = 0;
+  public:
+    explicit A(int x_) : x(x_) {}
+
+    int get_x() const {
+	return x;
+    }
+};
+
+class B : public Xapian::Internal::opt_intrusive_base {
+    int x = 0;
+    bool & alive;
+  public:
+    B(int x_, bool & alive_) : x(x_), alive(alive_) {
+	alive = true;
+    }
+
+    ~B() {
+	alive = false;
+    }
+
+    int get_x() const {
+	return x;
+    }
+
+    B * release() {
+	opt_intrusive_base::release();
+	return this;
+    }
+};
+
+static bool test_movesupport1()
+{
+    {
+	// Test move semantics support for intrusive_ptr class
+	Xapian::Internal::intrusive_ptr<A> p1(new A{5});
+	Xapian::Internal::intrusive_ptr<A> p3;
+
+	// Test move constructor
+	Xapian::Internal::intrusive_ptr<A> p2(std::move(p1));
+	TEST_EQUAL(p2->get_x(), 5);
+	TEST_EQUAL(p1.get(), 0);
+
+	// Test move assignment
+	p3 = std::move(p2);
+	TEST_EQUAL(p3->get_x(), 5);
+	TEST_EQUAL(p2.get(), 0);
+    }
+
+    bool alive = false;
+    {
+	// Same test for opt_intrusive_ptr class
+	B * b1 = new B{5, alive};
+	b1->release();
+	Xapian::Internal::opt_intrusive_ptr<B> p1(b1);
+	Xapian::Internal::opt_intrusive_ptr<B> p3;
+
+	// Test move constructor
+	Xapian::Internal::opt_intrusive_ptr<B> p2(std::move(p1));
+	TEST_EQUAL(p2->get_x(), 5);
+	TEST_EQUAL(p1.get(), 0);
+	TEST_EQUAL(alive, true);
+
+	// Test move assignment
+	p3 = std::move(p2);
+	TEST_EQUAL(p3->get_x(), 5);
+	TEST_EQUAL(p2.get(), 0);
+	TEST_EQUAL(alive, true);
+    }
+    // Test that object b1 has been deleted.
+    TEST_EQUAL(alive, false);
+    return true;
+}
+
+static bool test_addoverflows1()
+{
+    unsigned long res;
+    TEST(!add_overflows(0UL, 0UL, res));
+    TEST_EQUAL(res, 0);
+
+    TEST(add_overflows(ULONG_MAX, 1UL, res));
+    TEST_EQUAL(res, 0);
+
+    TEST(add_overflows(1UL, ULONG_MAX, res));
+    TEST_EQUAL(res, 0);
+
+    TEST(add_overflows(ULONG_MAX, ULONG_MAX, res));
+    TEST_EQUAL(res, ULONG_MAX - 1UL);
+
+    return true;
+}
+
+static bool test_muloverflows1()
+{
+    unsigned long res;
+    TEST(!mul_overflows(0UL, 0UL, res));
+    TEST_EQUAL(res, 0);
+
+    TEST(!mul_overflows(ULONG_MAX, 0UL, res));
+    TEST_EQUAL(res, 0);
+
+    TEST(!mul_overflows(0UL, ULONG_MAX, res));
+    TEST_EQUAL(res, 0);
+
+    TEST(!mul_overflows(ULONG_MAX, 1UL, res));
+    TEST_EQUAL(res, ULONG_MAX);
+
+    TEST(!mul_overflows(1UL, ULONG_MAX, res));
+    TEST_EQUAL(res, ULONG_MAX);
+
+    TEST(mul_overflows((ULONG_MAX >> 1UL) + 1UL, 2UL, res));
+    TEST_EQUAL(res, 0);
+
+    TEST(mul_overflows(2UL, (ULONG_MAX >> 1UL) + 1UL, res));
+    TEST_EQUAL(res, 0);
+
+    TEST(mul_overflows(ULONG_MAX, ULONG_MAX, res));
+
+    return true;
+}
+
 static const test_desc tests[] = {
     TESTCASE(simple_exceptions_work1),
     TESTCASE(class_exceptions_work1),
@@ -590,6 +796,10 @@ static const test_desc tests[] = {
     TESTCASE(tostring1),
     TESTCASE(strbool1),
     TESTCASE(closefrom1),
+    TESTCASE(uuid1),
+    TESTCASE(movesupport1),
+    TESTCASE(addoverflows1),
+    TESTCASE(muloverflows1),
     END_OF_TESTCASES
 };
 
